/*
**	Tamil to Phoneme rules.
**	
**	
**	1. Tamil characters are first transliterated using ASCII Mapping
**	
**	2. A Parser is used to differentiate Digit strings, abbreviations,etc.
**
**	3. Suitable normalization is done for each case.
**
**	4. Grapheme to Phoneme rules are applied on the parsed string.
**	
**	
**
**
**
**	
**  Tamil Transliteration Codes:
**
**	a		amma			N		kaN 
**	aa		aatu			th		thattu
**	i		ilai			nd		pandthu
**	ii		iikai			p		paatam
**	u		unavu			m		karumpu
**	uu		uur				y		yamunai
**	e		ettu			r		maram
**	ee		eeni			l		maalai
**	ai		aivar			v		vayathu
**	o		onRu			z		pazam 
**	oo		oom				L		uLLam 
**	au		auvai			R		aRam 
**	q		equ				n		ndaan
**	k		kalvi			Z		viZam 
**	ng		thangkam		h		hanumaan
**	c		cangku			j		jaathi
**	nj		njaanam			S		paSpam
**	t		attai			x		paxi
**								      
**									
**									
**			
**			
**
**	
**
**
**
**	Tamil Phoneme Codes:
**
**	ax   amma				t     attai
**       -					       -
**	aa   aatu			    d     katan
**	     -- 			            -
**	ih   ilai	            tx    thattu
**	     -					      --
**	iy   iikai				dh    pandtham
**	     --						      --
**	uh   unavu				T     mathi
**	     -						    --
**	U    thappu				p     paatam
**	          -					  -
**	uw   uur				b     ampu
**	     --					        -
**	ae   ettu				P     laapam
**	     -						     -
**	ey   eeni				m     karumpu
**	     --						      -
**	ao   onRu				v     vayathu
**	     -					      - 
**	ow   oom				y     yamunai
**	     --					      -
**	aw   auvai				r     maram
**	     --						    -
**	k    kalvi				tr    oRRan
**	     -					       --
**	g    pangku				dr    kanRu
**	         -						  -
**	hh   pakal				rr    aRam
**	       -					   -
**	ng   thangkam 			l     maalai
**			--			             -
**	nd   pandthu			ll    uLLam
**	       --		               -
**	n    ndaan				lzh   pazam
**	         - 		                -
**	ny   njaanam			sh    viZam
**		 --	    		            -
**	nn   kaN 				jh    jaathi
**		   -		              -
**  ch   kaatci             h     Identical Clusters except R
**		     -
**  s    cangku
**		 -
**  
**	
**  
**	
**	
**
**
**	Rules are made up of four parts:
**	
**		The left context.
**		The text to match.
**		The right context.
**		The phonemes to substitute for the matched text.
**
**	Procedure:
**
**		Seperate each block of letters (apostrophes included) 
**		and add a space on each side.  For each unmatched 
**		letter in the word, look through the rules where the 
**		text to match starts with the letter in the word.  If 
**		the text to match is found and the right and left 
**		context patterns also match, output the phonemes for 
**		that rule and skip to the next unmatched letter.
**
**
**	Special Context Symbols:
**	
**	
**		#	Consonant
**		@	Vowel
**		~   Mellinam (k,c,t,th,p,R)
**		%	Vallinam (ng,nj,N,nd,m,N)
**		^	Idayinam (y,r,l,v,z,L)
**		|	Retroflex (l,L,t,N)
**
**
**
*/


/* Context definitions */
static char Anything[] = "";	/* No context requirement */
static char Nothing[] = " ";	/* Context is beginning or end of word */

/* Phoneme definitions */
static char Pause[] = " ";	/* Short silence */
static char Silent[] = "";	/* No phonemes */

#define LEFT_PART	0
#define MATCH_PART	1
#define RIGHT_PART	2
#define OUT_PART	3

typedef char *Rule[4];	/* Rule is an array of 4 character pointers */

/*0 = Punctuation */
/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule punct_rules[] =
	{
	{Anything,	" ",		Anything,	Pause	},
	{Anything,	"-",		Anything,	Silent	},
	{Anything,	",",		Anything,	Pause	},
	{Anything,	".",		Anything,	Pause	},
	{Anything,	"?",		Anything,	Pause	},
	{Anything,	"!",		Anything,	Pause	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule A_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule B_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/
static Rule C_rules[] =
	{
	{Anything,	"C",		Anything,	"sh r i "	},
	{Anything,	0,			Anything,	Silent		},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule D_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule E_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule F_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule G_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule H_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule I_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule J_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule K_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/
static Rule L_rules[] =
	{
	{Anything,	"LL",		Anything,	"ll h "	},
	{Anything,	"L",		Anything,	"ll "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule M_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/
static Rule N_rules[] =
	{
	{Anything,	"NN",		Anything,	"nn h "	},
	{Anything,	"N",		Anything,	"nn "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule O_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule P_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule Q_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/
static Rule R_rules[] =
	{
	{Anything,	"RR",		Anything,	"tr "	},//eg.ndeRRu
	{"~",		"R",		Anything,	"dr "	},//eg.ndanRu
	{Anything,	"R",		Anything,	"rr "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/
static Rule S_rules[] =
	{
	{Anything,	"SS",		Anything,	"s h "	},
	{Anything,	"S",		Anything,	"s "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule T_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule U_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule V_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule W_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule X_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule Y_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/
static Rule Z_rules[] =
	{
	{Anything,	"ZZ",		Anything,	"sh h "	},
	{Anything,	"Z",		Anything,	"sh "	},//eg.kaZaayam
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/                                                   
static Rule a_rules[] =
	{
	{Anything,	"aa",		Anything,	"aa "	},
	{Anything,	"ai",		Anything,	"ax y "	},
	{Anything,	"au",		Anything,	"aw "	},
	{Anything,	"a",		Anything,	"ax "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule b_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/
static Rule c_rules[] =
	{
	{Anything,	"cc",		Anything,	"ch h "	},
	{Nothing,	"c",		Anything,	"ch*s "	},//eg.cennai
	{"#",		"c",		Anything,	"ch "	},//eg.muyaRci
	{Anything,	"c",			"#",	"ch "	},
	{Anything,	"c",		Anything,	"s "	},/*VOWEL c VOWEL*/
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule d_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART       /*CHANGED FOR TAMIL
*/
static Rule e_rules[] =
	{
	{Anything,	"ee",		Anything,	"ey "	},
	{Anything,	"e",		Anything,	"ae "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule f_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule g_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule h_rules[] =
	{
	{Anything,	"h",	Anything,	"hh "	},
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule i_rules[] =
	{
	{Anything,	"ii",		Anything,	"iy "	},
//	{Anything,	"i",		"|",		"U "	},//eg.piNi
	{Anything,	"i",		Anything,	"ih "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule j_rules[] =
	{
	{Anything,	"jj",		Anything,	"jh h "	},
	{Anything,	"j",		Anything,	"jh "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule k_rules[] =
	{
	/***************EXCEPTIONS*****************
	{Nothing,	"kuru",			Anything,	"g uh r uh "			},//eg.kuru,kurukripa
	{Nothing,	"keeth",		"@",		"g iy T "			},//eg.keethai,keeththa
	{Nothing,	"kaNapathi",	Anything,	"g ax nn ax p ax T ih "	},
	{Nothing,	"kaneeZ",		Anything,	"g aa n ey sh "			},
	{Nothing,	"kangkaa",		Anything,	"g ax ng g aa "			},
	{Nothing,	"kaayathri",	Anything,	"g aa y ax tx r ih "	},
	/***************EXCEPTIONS******************/

	{Nothing,	"k",		"@j",		"g "	},//eg.kajendthiran
	{Anything,	"kk",		Anything,	"k h "	},
	{"q",		"k",		Anything,	"h "	},//eg.eqku
	{Nothing,	"k",		Anything,	"k "	},//eg.katal
	{"~",		"k",		Anything,	"g "	},//eg.pangku
	{"^",		"k",		Anything,	"hh "	},//eg.nalku,avarkaL
	{"#",		"k",		Anything,	"k "	},//eg.iyaRkai
	{Anything,	"k",		"#",		"k "	},//eg.akpar
	{Anything,	"k",		Anything,	"hh "	}, /*VOWEL K VOWEL HH*///eg.varukirathu
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule l_rules[] =
	{
	{Anything,	"ll",		Anything,		"l h "	},
	{Anything,	"l",		Anything,		"l "	},
	{Anything,	0,			Anything,		Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule m_rules[] =
	{
	{Anything,	"mm",		Anything,	"m h "	},
	{Anything,	"m",		Anything,	"m "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule n_rules[] =
	{
	{Nothing,	"nd",		Anything,	"n "	},//eg.ndaatu
	{Anything,	"nd",			"th",	"nd "	},//eg.vandthu
	{Anything,	"nd",		Anything,	"n "	},//eg.thamizndaatu
	{Anything,	"njnj",		Anything,	"ny h "	},//eg.menjnjaanam
	{Nothing,	"nj",		Anything,	"ny "	},//eg.njaayiru
	{"#",		"nj",		Anything,	"ny "	},//eg.aknjaanam
	{Anything,	"njc",		Anything,	"ny jh "},//eg.manjcaL
	{Anything,	"nj",		Anything,	"ny "	},
	{Anything,	"ngng",		Anything,	"ng h "	},
	{Anything,	"ng",		Anything,	"ng "	},
	{Anything,	"n",			"nd",	"n "	},//eg.minndilaiyam
	{Anything,	"n",			"nj",	"n "	},
	{Anything,	"n",			"ng",	"n "	},
	{Anything,	"nn",		Anything,	"n h "	},
	{Anything,	"n",		Anything,	"n "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule o_rules[] =
	{
	{Anything,	"oo",		Anything,	"ow "	},
	{Anything,	"o",		Anything,	"ao "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule p_rules[] =
	{
	{Anything,	"pp",		Anything,	"p h "	},
	/***************EXCEPTIONS*****************
	{Nothing,	"pala",		Anything,	"*p b ax l ax "	},
	{Nothing,	"paava",	Anything,	"*p b aa v ax "	},
	{Nothing,	"pimpam",	Anything,	"b ih m b ax m "},
	{Nothing,	"pakthi",	Anything,	"b ax k tx i "	},
	{Nothing,	"pajan",	Anything,	"b ax jh ax n "	},
	/***************EXCEPTIONS******************/

	{Nothing,	"p",		Anything,	"p "		},
	{"~",		"p",		Anything,	"b "		},//eg.ampu
	{Anything,	"p",			"#",	"p "		},//eg.aptham
	{"#",		"p",		Anything,	"p "		},//eg.vetpam
	{Anything,	"p",		Anything,	"P "		},/*VOWEL p VOWEL*///eg.apayam
	{Anything,	0,			Anything,	Silent		},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule q_rules[] =
	{
	{Anything,	"q",		Anything,	"k "	},//eg.aqthu
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule r_rules[] =
	{
	{Anything,	"rr",		Anything,	"r h "	},
	{Anything,	"r",		Anything,	"r "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule s_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule t_rules[] =
	{
	{Anything,	"thth",		Anything,	"tx h "	},
	{Nothing,	"th",		Anything,	"tx "	},//eg.thattu
	{Anything,	"th",			"#",	"tx "	},//eg.aathmaa
	{"^",		"th",		Anything,	"T "	},//eg.neythal,vaalthal
	{"~",		"th",		Anything,	"dh "	},//eg.pandtham
	{"#",		"th",		Anything,	"tx "	},//eg.sulthaan
	{Anything,	"th",		Anything,	"T "	},/* VOWEL th VOWEL */
	{Anything,	"tc",		Anything,	"ch h "	},//eg.kaatci
	{Anything,	"t",			"th",	"t "	},
	{Anything,	"tt",		Anything,	"t h "	},
	{Nothing,	"t",		Anything,	"t "	},//eg.tin
	{Anything,	"t",			"#",	"t "	},//eg.vetpam
	{"~",		"t",		Anything,	"d "	},//eg.vaNti
	{"#",		"t",		Anything,	"t "	},//eg.kaStam
	{Anything,	"t",		Anything,	"d "	},/*VOWEL t VOWEL*/
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule u_rules[] =
	{
	{Anything,	"uu",		Anything,	"uw "	},
	{Nothing,	"u",		Anything,	"uh "	},//eg.ulakam
	{" #",		"u",		Anything,	"uh "	},//eg.kutam
	{Anything,	"u",		Nothing,	"U"		},//eg.athu
	{"|",		"u",		Anything,	"uh "	},//eg.siRRuNti
	{Anything,	"u",		"|",		"uh "	},//eg.ennutan
	{Anything,	"u",		Anything,	"U "	},//eg.vanthu
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule v_rules[] =
	{
	{Anything,	"vv",		Anything,	"v h "	},
	{Anything,	"v",		Anything,	"v "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule w_rules[] =
	{
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule x_rules[] =
	{
	{Anything,	"x",	Anything,	"k sh "	},
	{Anything,	0,		Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART      /*CHANGED FOR TAMIL
*/
static Rule y_rules[] =
	{
	{"ai",		"yy",		Anything,	"h "	}, //eg.aiyyoo
	{Anything,	"yy",		Anything,	"y h "	},
/*	{" #ai",	"y",		Anything,	"h "	}, //eg.kaiyaal
*/	{"ai",		"y",		Anything,	"h "	}, //eg.viLaiyaattu
	{Anything,	"y",		Anything,	"y "	},
	{Anything,	0,			Anything,	Silent	},
	};

/*
**	LEFT_PART	MATCH_PART	RIGHT_PART	OUT_PART
*/
static Rule z_rules[] =
	{
	{Anything,	"zz",		Anything,	"lzh h "	},
	{Anything,	"z",		Anything,	"lzh "		},
	{Anything,	0,			Anything,	Silent		},
	};


Rule *Rules[] =
	{
	punct_rules,
	A_rules, B_rules, C_rules, D_rules, E_rules, F_rules, G_rules, 
	H_rules, I_rules, J_rules, K_rules, L_rules, M_rules, N_rules, 
	O_rules, P_rules, Q_rules, R_rules, S_rules, T_rules, U_rules, 
	V_rules, W_rules, X_rules, Y_rules, Z_rules,
	a_rules, b_rules, c_rules, d_rules, e_rules, f_rules, g_rules, 
	h_rules, i_rules, j_rules, k_rules, l_rules, m_rules, n_rules, 
	o_rules, p_rules, q_rules, r_rules, s_rules, t_rules, u_rules, 
	v_rules, w_rules, x_rules, y_rules, z_rules
	};
