#include <stdio.h>
#include <ctype.h>

#define FALSE (0)
#define TRUE (!0)

/*
**	Tamil to Phoneme translation.
**
**	Rules are made up of four parts:
**	
**		The left context.
**		The text to match.
**		The right context.
**		The phonemes to substitute for the matched text.
**
**	Procedure:
**
**		Seperate each block of letters (apostrophes included) 
**		and add a space on each side.  For each unmatched 
**		letter in the word, look through the rules where the 
**		text to match starts with the letter in the word.  If 
**		the text to match is found and the right and left 
**		context patterns also match, output the phonemes for 
**		that rule and skip to the next unmatched letter.
**
**
**	Special Context Symbols:
**	
**	
**		#	Consonant
**		@	Vowel	  (a,aa,i,ii,u,uu,e,ee,ai,o,oo,au)
**		~   Vallinam  (k,c,t,th,p,R)
**		%	Mellinam  (ng,nj,N,nd,m,n)
**		^	Idayinam  (y,r,l,v,z,L)
**		|	Retroflex (l,L,t,N)
**
**
**
*/

typedef char *Rule[4];	/* A rule is four character pointers */

extern Rule *Rules[];	/* An array of pointers to rules */

int isvowel(chr)//(a,aa,i,ii,u,uu,e,ee,ai,o,oo,au)
	char chr;
	{
	return (chr == 'a' || chr == 'e' || chr == 'i' || chr == 'o' || chr == 'u');
	}

int isconsonant(chr)
	char chr;
	{
	return (!isvowel(chr));
	}

int isvallinam(chr)//(k,c,t,th,p,R)
	char chr;
	{
	return (chr == 'k' || chr == 'c' || chr == 't' || chr == 'h' || chr == 'p' || chr == 'R');
	}

int ismellinam(chr)//(ng,nj,N,nd,m,n)
	char chr;
	{
	return (chr == 'g' || chr == 'j' || chr == 'N'  || chr == 'd'  || chr == 'm'  || chr == 'n' );
	}

int isidayinam(chr)//(y,r,l,v,z,L)
	char chr;
	{
	return (chr == 'y' || chr == 'r' || chr == 'l' || chr == 'v' || chr == 'L' || chr == 'z');
	}

int isretroflex(chr)//(l,L,t,N)
	char chr;
	{
	return (chr == 'l' || chr == 'L' || chr == 't' || chr == 'N');
	}


xlate_word(word)
	char word[];
	{
	int index;	/* Current position in word */
	int type;	/* First letter of match part */

	index = 1;	/* Skip the initial blank */
	do
		{
		if (isalpha(word[index]))                       /*CHANGE*/
		{	type = word[index] - 'A' + 1;               /*CHANGE*/
			if (type > 26)								/*CHANGE*/
				type -= 6; 								/*CHANGE*/
		}
		else
			type = 0;

		index = find_rule(word, index, Rules[type]);
		}
	while (word[index] != '\0');
	}

find_rule(word, index, rules)
	char word[];
	int index;
	Rule *rules;
	{
	Rule *rule;
	char *left, *match, *right, *output;
	int remainder;

	for (;;)	/* Search for the rule */
		{
		rule = rules++;
		match = (*rule)[1];

		if (match == 0)	/* bad symbol! */
			{
			fprintf(stderr,"Error: Can't find rule for: '%c' in \"%s\"\n", word[index], word);
			return index+1;	/* Skip it! */
			}

		for (remainder = index; *match != '\0'; match++, remainder++)
			{
			if (*match != word[remainder])
				break;
			}

		if (*match != '\0')	/* found missmatch */
			continue;

		left = (*rule)[0];
		right = (*rule)[2];

		if (!leftmatch(left, &word[index-1]))
			continue;

		if (!rightmatch(right, &word[remainder]))
			continue;

		output = (*rule)[3];
	
		outstring(output);
		return remainder;
		}
	}


leftmatch(pattern, context)
	char *pattern;	/* first char of pattern to match in text */
	char *context;	/* last char of text to be matched */
	{
	char *pat;
	char *text;
	int count;

	if (*pattern == '\0')	/* null string matches any context */
		{
		return TRUE;
		}

	/* point to last character in pattern string */
	count = strlen(pattern);
	pat = pattern + (count - 1);

	text = context;

	for (; count > 0; pat--, count--)
		{
		/* First check for simple text or space */
		if (isalpha(*pat) || *pat == '\'' || *pat == ' ')
			if (*pat != *text)
				return FALSE;
			else
				{
				text--;
				continue;
				}

		switch (*pat)
			{
		case '#':	/* Consonant */
			if (isvowel(*text))
				return FALSE;

			text--;
			break;

		case '@':	/* Vowel */
			if (isconsonant(*text))
				return FALSE;

			text--;
			break;

		case '%':	/* Vallinam */
			if (!isvallinam(*text))
				return FALSE;
			
			text--;
			break;

		case '~':	/* Mellinam */
			if (!ismellinam(*text))
				return FALSE;
			
			text--;
			break;

		case '^':	/* Idayinam */
			if (!isidayinam(*text))
				return FALSE;
			
			text--;
			break;
		
		case '|':	/* Retroflex */
			if (!isretroflex(*text))
				return FALSE;
			
			text--;
			break;

		default:
			fprintf(stderr, "Bad char in left rule: '%c'\n", *pat);
			return FALSE;
			}
		}

	return TRUE;
	}


rightmatch(pattern, context)
	char *pattern;	/* first char of pattern to match in text */
	char *context;	/* last char of text to be matched */
	{
	char *pat;
	char *text;

	if (*pattern == '\0')	/* null string matches any context */
		return TRUE;

	pat = pattern;
	text = context;

	for (pat = pattern; *pat != '\0'; pat++)
		{
		/* First check for simple text or space */
		if (isalpha(*pat) || *pat == '\'' || *pat == ' ')
			if (*pat != *text)
				return FALSE;
			else
				{
				text++;
				continue;
				}

		switch (*pat)
			{
		case '#':	/* Consonant */
			if (isvowel(*text))
				return FALSE;

			text++;
			break;
		
		case '@':	/* Vowel */
			if (isconsonant(*text))
				return FALSE;

			text++;
			break;
		
		case '%':	/* Vallinam */
			if (!isvallinam(*text))
				return FALSE;

			text++;
			break;
		
		case '~':	/* Mellinam */
			if (!ismellinam(*text))
				return FALSE;

			text++;
			break;
		
		case '^':	/* Idayinam */
			if (!isidayinam(*text))
				return FALSE;

			text++;
			break;

		case '|':	/* Retroflex */
			if (!isretroflex(*text))
				return FALSE;

			text++;
			break;
				
		default:
			fprintf(stderr, "Bad char in right rule:'%c'\n", *pat);
			return FALSE;
			}
		}

	return TRUE;
	}
